"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.list = list;
exports.buildPrintData = buildPrintData;
exports.printDependencies = printDependencies;
exports.printDependenciesAsJson = printDependenciesAsJson;

var _path = _interopRequireDefault(require("path"));

var _os = _interopRequireDefault(require("os"));

var _dependencyGraph = require("../core/dependency-graph");

var _bundle = require("../core/bundle");

var _constant = require("../constant/constant");

var _repository = require("../core/repository");

var _config = require("../utils/config");

var _i18n = _interopRequireDefault(require("../i18n"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function list(_x, _x2) {
  return _list.apply(this, arguments);
}

function _list() {
  _list = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(options, args) {
    var bundle, repo, data;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            bundle = _bundle.Bundle.from((0, _config.getRootBundlePath)(options.global));
            repo = null;

            if (options.global) {
              repo = new _repository.Repository('', (0, _config.getGlobalRepo)());
            } else {
              repo = new _repository.Repository(_path["default"].join(process.cwd(), _constant.DEP_FOLDER_NAME), (0, _config.getGlobalRepo)());
            }

            if (!options.json) {
              _context.next = 7;
              break;
            }

            printDependenciesAsJson(bundle, repo);
            _context.next = 11;
            break;

          case 7:
            _context.next = 9;
            return buildPrintData(bundle, repo);

          case 9:
            data = _context.sent;
            printDependencies(data);

          case 11:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _list.apply(this, arguments);
}

function buildPrintData(_x3, _x4) {
  return _buildPrintData.apply(this, arguments);
}

function _buildPrintData() {
  _buildPrintData = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(bundle, repo) {
    var manifestGraph, result, traverse;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            traverse = function _traverse(parentManifest, parentLevel) {
              result.push({
                name: parentManifest.name,
                version: parentManifest.version.version,
                level: parentLevel
              });
              var manifests = manifestGraph.dependencies(parentManifest);
              manifests.forEach(function (m) {
                traverse(m, parentLevel + 1);
              });
            };

            _context2.next = 3;
            return _dependencyGraph.DependencyGraph.build(bundle, repo);

          case 3:
            manifestGraph = _context2.sent;
            result = [];
            traverse(bundle.manifest, 0);
            return _context2.abrupt("return", result);

          case 7:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));
  return _buildPrintData.apply(this, arguments);
}

function printDependencies(data) {
  var message = '';
  data.forEach(function (d) {
    return message += "".concat(''.padEnd(d.level * 2, '│ '), "+--").concat(d.name, "@").concat(d.version).concat(_os["default"].EOL);
  });

  _i18n["default"].log('common.message', {
    message: message
  });
}

function printDependenciesAsJson(_x5, _x6, _x7) {
  return _printDependenciesAsJson.apply(this, arguments);
}

function _printDependenciesAsJson() {
  _printDependenciesAsJson = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(bundle, repo, callByApi) {
    var manifestGraph, root, traverse;
    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            traverse = function _traverse2(parent, node) {
              node.name = parent.name;
              node.version = parent.version.version;
              node.manifest = parent.json();
              var manifests = manifestGraph.dependencies(parent);
              manifests.forEach(function (m) {
                var depNode = {};
                depNode.name = m.name;
                depNode.version = m.version.version;
                node.manifest = m.json();

                if (!node.dependencies) {
                  node.dependencies = [];
                }

                node.dependencies.push(depNode);
                traverse(m, depNode);
              });
            };

            _context3.next = 3;
            return _dependencyGraph.DependencyGraph.build(bundle, repo);

          case 3:
            manifestGraph = _context3.sent;
            root = {
              name: 1,
              version: 2
            };
            traverse(bundle.manifest, root);

            if (!callByApi) {
              console.log(JSON.stringify(root, null, 4));
            }

            return _context3.abrupt("return", root);

          case 8:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));
  return _printDependenciesAsJson.apply(this, arguments);
}