"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _crypto = _interopRequireDefault(require("crypto"));

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _helpers = require("../utils/helpers");

var _constant = require("../constant/constant");

var _log = _interopRequireDefault(require("./log"));

var _i18n = require("../i18n");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var Crypto = {
  digest: function digest(path) {
    var buffer = _fs["default"].readFileSync(path);

    var fsHash = _crypto["default"].createHash('sha256');

    fsHash.update(buffer);
    var digest = fsHash.digest('hex');
    return digest;
  },
  sign: function sign(hash, privateKey) {
    var sign = _crypto["default"].createSign('RSA-SHA256');

    sign.update(hash);
    return sign.sign(privateKey, 'hex');
  },
  getKey: function getKey(mode) {
    // mode: private | public
    var user = (0, _helpers.getLoginUser)();

    var folder = _path["default"].join(_constant.DEFAULT_CONFIG_DIR, 'key');

    var name = mode === 'private' ? 'privateKey' : 'publicKey';
    var fileSuffix = /[a-zA-Z0-9-_\u4e00-\u9fa5]+/g.exec(user);

    var key = _path["default"].join(folder, "".concat(name).concat(fileSuffix ? "_".concat(fileSuffix) : '', ".pem"));

    if (!_fs["default"].existsSync(key)) {
      throw (0, _i18n.getI18nMessage)('crypto.keymissing', {
        key: key
      });
    }

    _log["default"].info("read ".concat(mode, " key: ").concat(key));

    return _fs["default"].readFileSync(key);
  },
  generateKeyPair: function generateKeyPair(passphrase) {
    var user = (0, _helpers.getLoginUser)();

    var folder = _path["default"].join(_constant.DEFAULT_CONFIG_DIR, 'key');

    if (!_fs["default"].existsSync(folder)) {
      (0, _helpers.runShellCmd)(function (shell) {
        return shell.mkdir('-p', folder);
      });
    }

    var privateKeyEncoding = {
      type: 'pkcs8',
      format: 'pem'
    };

    if (passphrase) {
      privateKeyEncoding.cipher = 'aes-256-cbc';
      privateKeyEncoding.passphrase = passphrase;
    }

    var _crypto$generateKeyPa = _crypto["default"].generateKeyPairSync('rsa', {
      modulusLength: 4096,
      publicKeyEncoding: {
        type: 'spki',
        format: 'pem'
      },
      privateKeyEncoding: privateKeyEncoding
    }),
        publicKey = _crypto$generateKeyPa.publicKey,
        privateKey = _crypto$generateKeyPa.privateKey;

    var fileSuffix = /[a-zA-Z0-9-_\u4e00-\u9fa5]+/g.exec(user);

    var privatePath = _path["default"].join(folder, "privateKey".concat(fileSuffix ? "_".concat(fileSuffix) : '', ".pem"));

    var publicPath = _path["default"].join(folder, "publicKey".concat(fileSuffix ? "_".concat(fileSuffix) : '', ".pem"));

    _fs["default"].writeFileSync(privatePath, Crypto.aesEncrypt(privateKey), 'utf8');

    _log["default"].info("generate private key: ".concat(privatePath));

    _fs["default"].writeFileSync(publicPath, publicKey, 'utf8');

    _log["default"].info("generate public key: ".concat(publicPath));
  },
  privateEncrypt: function privateEncrypt(buffer, passphrase) {
    var privateKey = Crypto.getKey('private').toString('utf8');
    var opt = {
      key: Crypto.aesDecrypt(privateKey)
    };

    if (passphrase) {
      opt.passphrase = passphrase;
    }

    var encryptContent = _crypto["default"].privateEncrypt(opt, buffer);

    _log["default"].info('use private key encrypt');

    return encryptContent;
  },
  publicDecrypt: function publicDecrypt(buffer) {
    var publicKey = Crypto.getKey('public');

    var decryptContent = _crypto["default"].publicDecrypt({
      key: publicKey
    }, buffer);

    _log["default"].info('use public key decrypt');

    return decryptContent;
  },
  aesEncrypt: function aesEncrypt(data) {
    var mac = ((0, _helpers.getMacAddrs)()[0] || '').padEnd(24, 0);
    var nonce = Buffer.alloc(16, 0);

    var cipher = _crypto["default"].createCipheriv('aes192', mac, nonce);

    var encrypted = cipher.update(data, 'utf8', 'hex');
    encrypted += cipher["final"]('hex');
    return encrypted;
  },
  aesDecrypt: function aesDecrypt(data) {
    var nonce = Buffer.alloc(16, 0);
    var macs = (0, _helpers.getMacAddrs)();
    var decrypted = '';

    for (var i = 0; i < macs.length; i += 1) {
      try {
        var m = macs[i].padEnd(24, '0');

        var decipher = _crypto["default"].createDecipheriv('aes192', m, nonce);

        decrypted = decipher.update(data, 'hex', 'utf8');
        decrypted += decipher["final"]('utf8');

        if (decrypted) {
          break;
        }
      } catch (error) {
        decrypted = "";
      }
    }

    if (!decrypted) {
      throw (0, _i18n.getI18nMessage)('crypto.aesError');
    }

    return decrypted;
  }
};
var _default = Crypto;
exports["default"] = _default;