"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getTokenLoginCookie = getTokenLoginCookie;
exports.getCachedCookie = getCachedCookie;
exports.setCachedCookie = setCachedCookie;
exports.clearCachedCookie = clearCachedCookie;
exports.getCsrfToken = getCsrfToken;
exports.LoginType = void 0;

var _buffer = require("buffer");

var _path = _interopRequireDefault(require("path"));

var _fs = _interopRequireDefault(require("fs"));

var _inquirer = require("inquirer");

var _axios = _interopRequireDefault(require("../utils/axios"));

var _crypto = _interopRequireDefault(require("./crypto"));

var _helpers = require("../utils/helpers");

var _rcParser = _interopRequireDefault(require("./rc-parser"));

var _constant = require("../constant/constant");

var _i18n = require("../i18n");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var LoginType = {
  token: 'TOKEN',
  tty: 'TTY'
};
exports.LoginType = LoginType;

function getTokenLoginCookie(_x) {
  return _getTokenLoginCookie.apply(this, arguments);
}

function _getTokenLoginCookie() {
  _getTokenLoginCookie = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(loginUrl) {
    var cachedCookie, authParam, signature, inputJson, res, cookie;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            cachedCookie = getCachedCookie(LoginType.token);

            if (!cachedCookie) {
              _context.next = 3;
              break;
            }

            return _context.abrupt("return", cachedCookie);

          case 3:
            clearCachedCookie();
            authParam = {
              authorization: {
                user: (0, _helpers.getLoginUser)(),
                timestamp: new Date().getTime(),
                nonce: Math.floor(Math.random() * 1000000)
              }
            };
            _context.prev = 5;
            signature = _crypto["default"].privateEncrypt(_buffer.Buffer.from(JSON.stringify(authParam.authorization)));
            _context.next = 19;
            break;

          case 9:
            _context.prev = 9;
            _context.t0 = _context["catch"](5);

            if (!(_context.t0 && _context.t0.code === 'ERR_MISSING_PASSPHRASE')) {
              _context.next = 18;
              break;
            }

            _context.next = 14;
            return (0, _inquirer.prompt)([{
              type: 'password',
              name: 'pwd',
              message: (0, _i18n.getI18nMessage)('crypto.inputPassphrase'),
              mask: '*',
              validate: function validate(password) {
                try {
                  _crypto["default"].privateEncrypt(_buffer.Buffer.from(JSON.stringify(authParam.authorization)), password);
                } catch (error) {
                  return (0, _i18n.getI18nMessage)('crypto.passphraseError');
                }

                return true;
              }
            }]);

          case 14:
            inputJson = _context.sent;
            signature = _crypto["default"].privateEncrypt(_buffer.Buffer.from(JSON.stringify(authParam.authorization)), inputJson.pwd);
            _context.next = 19;
            break;

          case 18:
            throw _context.t0;

          case 19:
            authParam.signature = signature.toString('base64');
            authParam.authorization.pk = _crypto["default"].getKey('public').toString('utf8');
            _context.next = 23;
            return _axios["default"].get(loginUrl, {
              params: {
                authorization_code: JSON.stringify(authParam)
              }
            });

          case 23:
            res = _context.sent;

            if (!(res && res.headers['set-cookie'])) {
              _context.next = 29;
              break;
            }

            cookie = (res.headers['set-cookie'] || []).join(';');

            if (!cookie) {
              _context.next = 29;
              break;
            }

            setCachedCookie(cookie, LoginType.token);
            return _context.abrupt("return", cookie);

          case 29:
            throw res;

          case 30:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, null, [[5, 9]]);
  }));
  return _getTokenLoginCookie.apply(this, arguments);
}

function getCachedCookie(loginType) {
  var cookieFile = _path["default"].join(_constant.DEFAULT_CONFIG_DIR, 'user');

  if (_fs["default"].existsSync(cookieFile)) {
    var parser = _rcParser["default"].fromPath(cookieFile);

    var type = parser.get('type');

    if (type !== loginType) {
      return '';
    }

    var loginUser = (0, _helpers.getLoginUser)();
    var user = parser.get('user');

    if (user && loginUser && loginUser !== user) {
      return '';
    }

    var cookie = parser.get('cookie');
    return cookie;
  }

  return '';
}

function setCachedCookie(cookie, loginType) {
  var cookieFile = _path["default"].join(_constant.DEFAULT_CONFIG_DIR, 'user');

  var json = {
    user: (0, _helpers.getLoginUser)(),
    type: loginType,
    cookie: cookie
  };

  _rcParser["default"].saveJsonToFile(json, cookieFile);
}

function clearCachedCookie() {
  (0, _helpers.runShellCmd)(function (shell) {
    return shell.rm('-rf', _path["default"].join(_constant.DEFAULT_CONFIG_DIR, 'user'));
  });
}

function getCsrfToken(cookie) {
  var cookieList = cookie.split(';').map(function (c) {
    return c.trim();
  });
  var csrfTokenMark = 'hpmCsrfToken=';
  var csrfToken = cookieList.find(function (c) {
    return c.startsWith(csrfTokenMark);
  });

  if (csrfToken) {
    return csrfToken.substr(csrfTokenMark.length);
  }

  return '';
}