"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.template = exports.parseTemplate = void 0;

var _tar = _interopRequireDefault(require("tar"));

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _helpers = require("../utils/helpers");

var _helpers2 = require("./helpers");

var _remoteRepository = require("./remote-repository");

var _i18n = require("../i18n");

var _constant = require("../constant/constant");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var parseTemplate = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(templatename, dirname) {
    var defaultTemplateFolder, dirs, repo, trimed, index, templateVersion, templateParseName, manifest, templateDirPath;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (!templatename) {
              _context.next = 5;
              break;
            }

            defaultTemplateFolder = _path["default"].join(__dirname, '..', 'asset', 'template', 'default');
            dirs = _fs["default"].readdirSync(defaultTemplateFolder);

            if (!dirs.includes("".concat(templatename, ".tgz"))) {
              _context.next = 5;
              break;
            }

            return _context.abrupt("return", template(templatename, dirname));

          case 5:
            repo = new _remoteRepository.RemoteRepository();
            trimed = templatename.trim();
            index = trimed.lastIndexOf('@');
            templateVersion = (index > 0 ? trimed.substr(index + 1) : '') || '*'; // index === 0 is scoped name

            templateParseName = index > 0 ? trimed.substr(0, index).trim() : trimed;
            _context.next = 12;
            return repo.manifest(templateParseName, templateVersion);

          case 12:
            manifest = _context.sent;

            if (manifest) {
              _context.next = 15;
              break;
            }

            throw (0, _i18n.getI18nMessage)('template.notFound', {
              name: templatename
            });

          case 15:
            if (!(manifest.publishAs !== 'template')) {
              _context.next = 17;
              break;
            }

            throw (0, _i18n.getI18nMessage)('template.isNotTemplate', {
              name: templateParseName
            });

          case 17:
            templateDirPath = _path["default"].join(_constant.DEFAULT_CONFIG_DIR, 'template');

            if (!_fs["default"].existsSync(templateDirPath)) {
              (0, _helpers.runShellCmd)(function (shell) {
                return shell.mkdir('-p', templateDirPath);
              });
            }

            _context.next = 21;
            return (0, _helpers2.download)(manifest, templateDirPath);

          case 21:
            return _context.abrupt("return", template(manifest.id, dirname, true));

          case 22:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function parseTemplate(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

exports.parseTemplate = parseTemplate;

var template = function template(name) {
  var dirname = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : process.cwd();
  var isRemote = arguments.length > 2 ? arguments[2] : undefined;
  var manifestJson = {};
  var templatePath = isRemote ? _path["default"].join(_constant.DEFAULT_CONFIG_DIR, 'template', "".concat(name, ".tgz")) : _path["default"].join(__dirname, '..', 'asset', 'template', 'default', "".concat(name, ".tgz"));

  if (_fs["default"].existsSync(templatePath)) {
    _tar["default"].extract({
      file: templatePath,
      cwd: dirname,
      sync: true
    });
  }

  if (_fs["default"].existsSync(_path["default"].join(dirname, 'bundle.json'))) {
    var bundle = _fs["default"].readFileSync(_path["default"].join(dirname, 'bundle.json'));

    manifestJson = JSON.parse(bundle.toString());
    (0, _helpers.runShellCmd)(function (shell) {
      return shell.rm(_path["default"].join(dirname, 'bundle.json'));
    });
  }

  return {
    templatePath: templatePath,
    manifestJson: manifestJson
  };
};

exports.template = template;