const os = require('os');
const https = require('https');
const URL = require('url');
const http = require('http');
const fs = require('fs');
const path = require('path');
const execSync = require('child_process').execSync;
function readJSONFile(filePath) {
    return JSON.parse(fs.readFileSync(filePath).toString())
}
function download({ url, envNameSpace }) {
    return new Promise((resolve, reject) => {

        let agent = http;

        if (URL.parse(url).protocol === 'https:') {
            agent = https;
        }
        console.log(`request: ${url}`);
        agent.get(url, (res) => {
            let filename = getfilename(res, url);
            const file = fs.createWriteStream(path.resolve(filename));
            console.log(`Downloading from: ${url}`);
            res.on('data', function (data) {
                file.write(data);
            }).on('end', function () {
                file.end();
                console.log('Download completed')
                file.on('close', function () {
                    resolve({ filename, envNameSpace })
                });
            }).on('error', function (err) {
                reject(err);
            });
        });
    })

}
function getfilename(res, url) {
    const infoString = res.headers["content-disposition"] || res.headers["Content-Disposition"] || '';

    if (infoString && infoString.includes('filename=')) {
        return infoString.split('filename=')[1].replace(/"|'/g, '');
    }
    const list = url.split('/');
    return list[list.length - 1].split("?")[0]

}

function getInstallUrl() {
    const bundleJson = readJSONFile(path.resolve('bundle.json'));

    const { platform: { linux, win, darwin }, envNameSpace } = bundleJson.installConfig;

    let installUrl = win
    switch (os.platform()) {
        case "darwin": {
            installUrl = darwin
            break;
        }
        case "linux": {
            installUrl = linux
            break;
        }
        default:
            break;
    }

    if (!installUrl || !envNameSpace) {
        throw `install config in bundle.json need at least one download url (win,linux or darwin),and envNameSpace is required`
    }
    return { url: installUrl, envNameSpace };
}

download(getInstallUrl()).then(({ filename, envNameSpace }) => {
    let extractPath = process.env[envNameSpace];
    // 若指定环境变量的值为空则将其指定为当前bundle目录的值并添加至 .hpmrc中
    if (!extractPath) {
        extractPath = path.resolve('.');
        console.log(`Adding config: ${envNameSpace}`)
        execSync(`hpm config set ${envNameSpace} ${extractPath}`, { stdio: 'inherit' });
    }
    console.log('Start extract file ...')
    console.log(`Extract file to "${extractPath}"`);
    execSync(`hpm extract ${filename} ${extractPath}`);
    console.log(`Extract complated`);
    execSync(`rm ${filename}`)
}).catch(() => {
    'Download failed'
});


